<?php

/*
 * SePay - Quét QR Code chuyển khoản
 * Description: Giải pháp tự động xác nhận thanh toán chuyển khoản ngân hàng
 * Version: 1.4
 * Author: Vu Quang Son
 */
class sepay extends PaymentModule {
    protected $modname = 'SePay - Quét QR Code chuyển khoản';
    protected $description = 'SePay - Giải pháp tự động xác nhận thanh toán chuyển khoản ngân hàng';
	protected $supportedCurrencies = array('VND');
    protected $version = "1.3";
    protected $configuration = array(
        'bank_name' => array(
            'type' => 'multiselect',
            'default' => array(
                'TPBank',
                'ACB',
                'VietinBank',
                'MBBank',
                'OCB',
                'KienLongBank',
                'BIDV',
                'MSB',
                'Vietcombank',
                'VPBank',
                'Sacombank',
                'HDBank',
                'Techcombank',
                'ShinhanBank',
                'Eximbank',
                'VIB',
                'Agribank',
                'PBVN',
                'ABBank',
                'LienVietPostBank',
                'BacABank',
                'SeABank',
                'VietCapitalBank',
                'SHB',
                'NCB',
                'Woori',
                'VietABank',
                'VietBank',
                'NamABank',
                'PGBank',
            ),
        ),
        'bank_info' => array(
            'type' => 'textarea',
            'value' => '',
            'description' => 'Enter bank details, each line in the format: bank_name|bank_account_name|bank_number|min_price|max_price'
        ),
        'accept_trans_lower_amount' => array(
            'type' => 'check',
            'value' => '',
            'description' => 'Allow adding transactions to invoice when payment amount is less than invoice amount.'
        ),
        'accept_trans_higher_amount' => array(
            'type' => 'check',
            'value' => '',
            'description' => 'Allow adding transactions to invoice when payment amount is greater than invoice amount.'
        ),
        'payment_code_prefix' => array(
            'type' => 'input',
            'value' => '',
            'description' => 'The transaction content prefix is configured on SePay to accept transactions.'
        ),
        'template_qr' => array(
            'type' => 'select',
            'default' => array(
                'compact' => 'Khung VietQR (compact)',
                '' => 'Hiện mã QR kèm logo V',
                'qronly' => 'Chỉ hiện mã QR (qronly)',
            ),
            'description' => 'QR code template displayed on the payment interface.'
        ),
        'api_key' => array(
            'type' => 'input',
            'value' => '',
            'description' => 'A randomly generated key that matches the authentication key configured on Sepay.'
        ),
        'time_interval' => array(
            'type' => 'input',
            'value' => '3',
            'description' => 'Allows you to customize the time interval for the system to check the invoice status. The time interval is limited to a range of 1 second to 10 seconds.'
        ),
    );
    protected $lang = array(
        'english' => array(
            'bank_name' => 'Bank Name',
            'bank_number' => 'Bank Number',
            'bank_info' => 'Bank Info',
            'va_number' => 'VA Number',
            'bank_account_name' => 'Bank Account Name',
            'payment_code_prefix' => 'Payment Code Prefix',
            'api_key' => 'API Key',
            'template_qr' => 'Template QR',
            'accept_trans_lower_amount' => 'Allow adding transactions to invoice when payment amount is less than invoice amount',
            'accept_trans_higher_amount' => 'Allow adding transactions to invoice when payment amount is greater than invoice amount',
            'time_interval' => 'Periodic invoice status check interval (s)',
        )
    );

    private function fetchBankList() {
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, "https://qr.sepay.vn/banks.json");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
        ]);

        $response = curl_exec($ch);

        if(curl_errno($ch)) {
            echo 'Error:' . curl_error($ch);
        }

        curl_close($ch);

        if ($response) {
            return json_decode($response, true)["data"];
        } else {
            return [];
        }
    }

    function drawForm($autosubmit = false) {
        $lang = new Language();
        $lang->loadDictionary();
        $this->form = "";

        $api = new ApiWrapper();
		$invoiceRes = $api->getInvoiceDetails(['id' => $this->invoice_id]);
        $bankLines = explode("\n", $this->configuration['bank_info']['value']);

        $invoice_id = $this->invoice_id;
        $proforma_id = $invoiceRes['invoice']['proforma_id'] ?? $invoice_id;
        $amount = $this->amount;
        $banks = $this->fetchBankList();

        $filteredBanks = array_map(function($line) use ($amount) {
            $parts = explode('|', trim($line));
            $minPrice = trim($parts[3]);
            $maxPrice = trim($parts[4]);
            $bankName = trim($parts[0] ?? null);
        
            if (!in_array($this->stripBankCode($bankName), array_map([$this, 'stripBankCode'], $this->configuration['bank_name']['value'])) ||
                (is_numeric($minPrice) && $amount < (float) $minPrice) || 
                (is_numeric($maxPrice) && $amount > (float) $maxPrice)) {
                return null;
            }
        
            return [
                'bank_name' => trim($parts[0] ?? ""),
                'bank_account_name' => trim($parts[1] ?? ""),
                'bank_number' => trim($parts[2] ?? ""),
                'min_price' => $minPrice,
                'max_price' => $maxPrice,
            ];
        }, $bankLines);
        
        $filteredBanks = array_values(array_filter($filteredBanks));
        if (count($filteredBanks) > 0) {
            $timeInterval = $this->configuration['time_interval']['value'] ?? 3;
            if (!is_numeric($timeInterval) || (int)$timeInterval != $timeInterval || $timeInterval < 1 || $timeInterval > 10) {
                $timeInterval = 3;
            } else {
                $timeInterval = (int)$timeInterval;
            }
            $timeInterval *= 1000;

            $hiddenBelow = count($filteredBanks) == 1 ? 'hidden-block' : '';
            $this->form .= "<style>.hidden-block{display: none !important;} .bank-select:hover{border:1px solid green !important} .bank-select:has(input[type=radio]:checked) {position: relative;border:1px solid green !important}</style>";
            $this->form .= "<div class='$hiddenBelow' style='text-align: center; margin:8px 0;font-weight:700'>Chọn ngân hàng thanh toán</div>";
            $this->form .= "<div class='$hiddenBelow' style='display:flex;flex-wrap:wrap;justify-content:center;gap:4px;'>";
            foreach ($filteredBanks as $key => $bank) {
                $bankInfo = reset(array_filter($banks, function($item) use ($bank) {
                    return $this->stripBankCode($item['short_name']) == $this->stripBankCode($bank["bank_name"]);
                }));

                $bankName = $bankInfo['name'] ?? $bank['bank_name'];
                $bankShortName = $bankInfo['short_name'] ?? $bank['bank_name'];
                $bankCode = $bankInfo['code'] ?? $bank['bank_name'];
                $bankNumber = $bank['bank_number'];
                $bankAccountName = $bank['bank_account_name'];
                $paymentCodePrefix = trim($this->configuration['payment_code_prefix']['value']);

                $bankDesSrc = $this->stripBankCode($bankShortName) == $this->stripBankCode("VietinBank") ? "SEVQR+{$paymentCodePrefix}{$proforma_id}" : "{$paymentCodePrefix}{$proforma_id}";
                $bankDes = str_replace('+', ' ', $bankDesSrc);

                preg_match('/\((.*?)\)/', $this->configuration['template_qr']['value'], $matches);
                $templateQR = $matches[1] ?? '';

                $srcQR = "https://qr.sepay.vn/img?bank={$bankShortName}&acc={$bankNumber}&amount={$amount}&des=$bankDesSrc&template=$templateQR";
                $srcQRDownload = $srcQR . "&download=yes";

                $active = $key == 0 ? "checked" : "";
                $this->form .= "<label class='bank-select' style='border: 1px dashed #ccc;border-radius:8px;padding: 4px 8px;cursor:pointer;'>
                                    <input type='radio' name='bank-select' $active hidden 
                                        data-bank-name='{$bankName}'
                                        data-bank-short-name='{$bankShortName}'
                                        data-bank-code='{$bankCode}'
                                        data-bank-number='{$bankNumber}'
                                        data-bank-account-name='{$bankAccountName}'
                                        data-payment-code-prefix='{$paymentCodePrefix}'
                                        data-src-qr='{$srcQR}'
                                        data-src-qr-download='{$srcQRDownload}'
                                        data-trans-amount='{$amount}'
                                        data-trans-des='{$bankDes}'
                                        />
                                    <img style='height: 40px;' src='https://qr.sepay.vn/assets/img/banklogo/{$bankInfo['code']}.png' alt=''>
                                </label>";
            }
            $this->form .= "</div>";

            $this->form .= "<div style='text-align: center; margin:16px 0 8px;'>Mở app ngân hàng và quét mã QR bên dưới</div>";
            $this->form .= "<div style='text-align: center;'><img id='src-qr' style='width: 80%;max-width: 260px;' alt=''></div><br>";
            $this->form .= "<div style='text-align: center;'><a id='src-qr-download' style='border: 1px solid #3891F2;padding: 4px 8px;' target='_blank' download>Tải ảnh QR</a></div><br>";
            $this->form .= "<div style='text-align: center; margin-top: 16px;'>Hoặc chuyển khoản thủ công theo thông tin bên dưới</div>";
            $this->form .= "<table style='margin: 0 auto;'>
                    <tbody>
                        <tr>
                            <td colspan='2' style='text-align: center;'><img id='src-bank-logo' style='max-height: 52px;' alt=''></td>
                        </tr>
                        <tr>
                            <td id='bank-name' colspan='2' style='text-align: center;font-weight: 700'></td>
                        </tr>
                        <tr>
                            <td>Chủ tài khoản:</td>
                            <td><span id='bank-account-name' style='font-weight: 700'></span></td>
                        </tr>
                        <tr>
                            <td>Số tài khoản:</td>
                            <td><span id='bank-number' style='font-weight: 700'></td>
                        </tr>
                        <tr>
                            <td>Số tiền:</td>
                            <td><span id='trans-amount' style='font-weight: 700'></td>
                        </tr>
                        <tr>
                            <td>Nội dung:</td>
                            <td><span id='trans-des' style='font-weight: 700'></td>
                        </tr>
                    </tbody>
                </table>";

            $this->form .= "<script language='javascript'>
                    document.querySelectorAll('input[name=bank-select]').forEach(function(input) {
                        input.addEventListener('change', function() {
                            const bankName = this.getAttribute('data-bank-name');
                            const bankShortName = this.getAttribute('data-bank-short-name');
                            const bankCode = this.getAttribute('data-bank-code');
                            const bankNumber = this.getAttribute('data-bank-number');
                            const bankAccountName = this.getAttribute('data-bank-account-name');
                            const paymentCodePrefix = this.getAttribute('data-payment-code-prefix');
                            const srcQR = this.getAttribute('data-src-qr');
                            const srcQRDownload = this.getAttribute('data-src-qr-download');
                            const transAmount = this.getAttribute('data-trans-amount');
                            const transDes = this.getAttribute('data-trans-des');
                            
                            
                            document.getElementById('src-qr').src = srcQR;
                            document.getElementById('src-qr-download').href = srcQRDownload;
                            document.getElementById('src-bank-logo').src = 'https://qr.sepay.vn/assets/img/banklogo/' + bankCode + '.png';
                            document.getElementById('bank-name').innerHTML = bankName + '<br/>' + '(' + bankShortName + ')';
                            document.getElementById('bank-number').textContent = bankNumber;
                            document.getElementById('bank-account-name').textContent = bankAccountName;
                            document.getElementById('trans-amount').textContent = transAmount;
                            document.getElementById('trans-des').textContent = transDes;
                        });
                    });
                    document.querySelector('input[name=bank-select]').dispatchEvent(new Event('change'));
                    
                    function check_invoice_status() {
                        $.getJSON('?cmd=sepay&action=invoiceStatus&invoice_id=$invoice_id', function(data) {
                            if (data.status == 'Paid') {
                                location.reload();
                            }
                        });
                    }

                    setInterval(function() {
                        check_invoice_status();
                    }, $timeInterval);
                </script>
                ";

            if ($autosubmit) {
                $this->form .= '<form name=\'payform\' method="post" action="?cmd=clientarea&action=invoice&id=' . $invoice_id . '"></form>
                <script language="javascript">
                setTimeout ( "autoForward()" , 5000 );
                function autoForward() {
                    document.forms.payform.submit()
                }
                </script>
                ';
            }
        }

		return $this->form;
    }

    function callback() {
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $apiKey = trim($this->configuration['api_key']['value']);
            if (strlen($apiKey) > 0) {
                $headers = getallheaders();
                $authorizationHeader = isset($headers['Authorization']) ? $headers['Authorization'] : '';
                $authorizationKey = str_replace('Apikey ', '', $authorizationHeader);

                if ($apiKey != $authorizationKey) {
                    $this->sendResponse(false, 401, "Invalid or missing Authorization header.");
                    exit;
                }
            }
           
            $rawBody = file_get_contents('php://input');
            $parsedBody = json_decode($rawBody, true);
            if (json_last_error() !== JSON_ERROR_NONE) {
                $this->sendResponse(false, 400, "Invalid JSON payload.");
                exit;
            }

            $codeRes = trim($parsedBody['code']);
            $codePrefixModule = trim($this->configuration['payment_code_prefix']['value']);


            try {
                $query = $this->db->prepare("SELECT cf.`value` FROM `hb_configuration` cf WHERE cf.`setting` = 'InvoicePrefix'");
                $query->execute();
                $codePrefixConfig = trim($query->fetch(PDO::FETCH_ASSOC)['value'] ?? '');
            } catch (PDOException $e) {
                $this->sendResponse(false, 500, "Database error: " . $e->getMessage());
                exit;
            }

            $invoiceID = str_replace($codePrefixModule . $codePrefixConfig, '', $codeRes);
            $transferAmount = trim($parsedBody['transferAmount']);
            $transactionID = trim($parsedBody['id']);
            
            if($this->_transactionExists($transactionID)) {
                $this->sendResponse(false, 409, "Transaction already exists.");
                exit;
            }

            $acceptTransLowerAmount = !empty($this->configuration['accept_trans_lower_amount']['value']);
            $acceptTransHigherAmount = !empty($this->configuration['accept_trans_higher_amount']['value']);

            $api = new ApiWrapper();
            $invoice = $api->getInvoiceDetails(['id' => $invoiceID]);
            if (!$invoice || !$invoice['success']) {
                $this->sendResponse(false, 404, "Invalid transfer content.");
                exit;
            }

            if (!$acceptTransLowerAmount || !$acceptTransHigherAmount) {
                $invoiceTotal = (float)$invoice['invoice']['total'];
                if (!$acceptTransLowerAmount && ($transferAmount < $invoiceTotal)) {
                    $this->sendResponse(false, 400, "The transaction amount is lower than the invoice value.");
                    exit;
                }

                if (!$acceptTransHigherAmount && ($transferAmount > $invoiceTotal)) {
                    $this->sendResponse(false, 400, "The transaction amount is higher than the invoice value.");
                    exit;
                }
            }

            $response = $api->addTransaction(array(
                'in' => $transferAmount,
                'invoice_id' => $invoiceID,
                'trans_id' => $transactionID,
                'module' => $this->id,
            ));

            if ($response) {
                $this->sendResponse(true, 201, "Transaction added successfully.");
                $this->logActivity(array(
                    'result' => 'Successfull',
                    'output' => $parsedBody
                ));
            } else {
                $this->sendResponse(false, 400, $response);
            }
            exit;
        }
    }

    private function sendResponse($status, $code, $message) {
        $response = [
            "success" => $status,
            "message" => $message,
        ];

        http_response_code($code);
        header('Content-Type: application/json');

        echo json_encode($response);
    }

    function invoiceStatus($request) {
        $client_info = hbm_logged_client();
		if($client_info !== FALSE && isset($request['invoice_id']) && is_numeric($request['invoice_id'])) {
			if(isset($client_info['id']) && is_numeric($client_info['id']) && $client_info['id'] > 0) {
				return $this->getInvoiceStatus($request['invoice_id'], $client_info['id']); 
			}
		}
		return 'Unknown';
    }

    private function getInvoiceStatus($invoice_id, $client_id) {
        $query = $this->db->prepare("SELECT status from hb_invoices where id=? AND client_id=?");
        $query->execute(array($invoice_id, $client_id));
        $result = $query->fetch(PDO::FETCH_ASSOC);
        if(is_array($result) && isset($result['status']))
            return $result['status'];
        return FALSE;
    }

    private function stripBankCode($str) {
        $str = preg_replace("/(à|á|ạ|ả|ã|â|ầ|ấ|ậ|ẩ|ẫ|ă|ằ|ắ|ặ|ẳ|ẵ)/", 'a', $str);
        $str = preg_replace("/(è|é|ẹ|ẻ|ẽ|ê|ề|ế|ệ|ể|ễ)/", 'e', $str);
        $str = preg_replace("/(ì|í|ị|ỉ|ĩ)/", 'i', $str);
        $str = preg_replace("/(ò|ó|ọ|ỏ|õ|ô|ồ|ố|ộ|ổ|ỗ|ơ|ờ|ớ|ợ|ở|ỡ)/", 'o', $str);
        $str = preg_replace("/(ù|ú|ụ|ủ|ũ|ư|ừ|ứ|ự|ử|ữ)/", 'u', $str);
        $str = preg_replace("/(ỳ|ý|ỵ|ỷ|ỹ)/", 'y', $str);
        $str = preg_replace("/(đ)/", 'd', $str);
    
        $str = preg_replace("/(À|Á|Ạ|Ả|Ã|Â|Ầ|Ấ|Ậ|Ẩ|Ẫ|Ă|Ằ|Ắ|Ặ|Ẳ|Ẵ)/", 'A', $str);
        $str = preg_replace("/(È|É|Ẹ|Ẻ|Ẽ|Ê|Ề|Ế|Ệ|Ể|Ễ)/", 'E', $str);
        $str = preg_replace("/(Ì|Í|Ị|Ỉ|Ĩ)/", 'I', $str);
        $str = preg_replace("/(Ò|Ó|Ọ|Ỏ|Õ|Ô|Ồ|Ố|Ộ|Ổ|Ỗ|Ơ|Ờ|Ớ|Ợ|Ở|Ỡ)/", 'O', $str);
        $str = preg_replace("/(Ù|Ú|Ụ|Ủ|Ũ|Ư|Ừ|Ứ|Ự|Ử|Ữ)/", 'U', $str);
        $str = preg_replace("/(Ỳ|Ý|Ỵ|Ỷ|Ỹ)/", 'Y', $str);
        $str = preg_replace("/(Đ)/", 'D', $str);
        
        return str_replace(' ','',strtolower(trim($str)));
    }
}